package com.yaoyicloud.easier.common.data.datascope;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.springframework.security.core.GrantedAuthority;

import com.yaoyicloud.easier.common.core.constant.SecurityConstants;
import com.yaoyicloud.easier.common.core.util.RetOps;
import com.yaoyicloud.easier.common.domain.enums.type.CommonType;
import com.yaoyicloud.easier.common.security.service.EasierUser;
import com.yaoyicloud.easier.common.security.util.SecurityUtils;
import com.yaoyicloud.easier.workspace.api.entity.SysDept;
import com.yaoyicloud.easier.workspace.api.entity.SysRole;
import com.yaoyicloud.easier.workspace.api.feign.RemoteDataScopeService;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;

/**
 * 默认data scope 判断处理器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2022-12-02 18:41
 */
@RequiredArgsConstructor
public class EasierDefaultDatascopeHandle implements DataScopeHandle {

    private final RemoteDataScopeService dataScopeService;

    /**
     * 计算用户数据权限
     *
     * @param dataScope 数据权限范围
     * @return 计算结果
     */
    @SuppressWarnings({"checkstyle:ReturnCount", "checkstyle:WhitespaceAround"})
    @Override
    public Boolean calcScope(DataScope dataScope) {
        EasierUser user = SecurityUtils.getUser();
        // toc 客户端不进行数据权限
        if (CommonType.UserType.TO_C.equals(user.getUserType())) {
            return true;
        }

        List<String> roleIdList = user.getAuthorities().stream().map(GrantedAuthority::getAuthority)
            .filter(authority -> authority.startsWith(SecurityConstants.ROLE))
            .map(authority -> authority.split(StrUtil.UNDERLINE)[1]).collect(Collectors.toList());

        List<Long> deptList = dataScope.getDeptList();

        // 当前用户的角色为空 , 返回false
        if (CollectionUtil.isEmpty(roleIdList)) {
            return false;
        }
        // @formatter:off
		SysRole role = RetOps.of(dataScopeService.getRoleList(roleIdList))
				.getData()
				.orElseGet(Collections::emptyList)
				.stream()
				.min(Comparator.comparingInt(r-> r.getDataScopeType().getCode())).orElse(null);
		// @formatter:on
        // 角色有可能已经删除了
        if (role == null) {
            return false;
        }
        CommonType.DataScopeType dsType = role.getDataScopeType();
        // 查询全部
        if (Objects.equals(dsType, CommonType.DataScopeType.ALL)) {
            return true;
        }
        // 自定义
        if (Objects.equals(dsType, CommonType.DataScopeType.CUSTOM) && ArrayUtil.isNotEmpty(role.getScopeIds())) {
            Long[] scopeIds = role.getScopeIds();
            deptList.addAll(List.of(scopeIds));
        }
        // 查询本级及其下级
        if (Objects.equals(dsType, CommonType.DataScopeType.SET_AND_SUBSET)) {
            // @formatter:off
			List<Long> deptIdList = RetOps.of(dataScopeService.getDescendantList(user.getDeptId()))
					.getData()
					.orElseGet(Collections::emptyList)
					.stream()
					.map(SysDept::getDeptId).collect(Collectors.toList());
			// @formatter:on
            deptList.addAll(deptIdList);
        }
        // 只查询本级
        if (Objects.equals(dsType, CommonType.DataScopeType.SET)) {
            deptList.add(user.getDeptId());
        }

        // 只查询本人
        if (Objects.equals(dsType, CommonType.DataScopeType.SELF)) {
            dataScope.setUsername(user.getUsername());
        }
        return false;
    }

}
