package com.yaoyicloud.easier.common.data.mybatis;

import java.util.List;
import java.util.Properties;

import javax.sql.DataSource;

import org.apache.ibatis.mapping.DatabaseIdProvider;
import org.apache.ibatis.mapping.VendorDatabaseIdProvider;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.OptimisticLockerInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.yaoyicloud.easier.common.data.application.EasierApplicationConfigProperties;
import com.yaoyicloud.easier.common.data.application.EasierApplicationHandler;
import com.yaoyicloud.easier.common.data.datascope.DataScopeInnerInterceptor;
import com.yaoyicloud.easier.common.data.datascope.DataScopeInterceptor;
import com.yaoyicloud.easier.common.data.datascope.DataScopeSqlInjector;
import com.yaoyicloud.easier.common.data.datascope.EasierDefaultDatascopeHandle;
import com.yaoyicloud.easier.common.data.handler.JsonLongArrayTypeHandler;
import com.yaoyicloud.easier.common.data.handler.JsonStringArrayTypeHandler;
import com.yaoyicloud.easier.common.data.resolver.SqlFilterArgumentResolver;
import com.yaoyicloud.easier.common.data.tenant.EasierTenantConfigProperties;
import com.yaoyicloud.easier.common.data.tenant.EasierTenantHandler;
import com.yaoyicloud.easier.common.security.service.EasierUser;
import com.yaoyicloud.easier.workspace.api.feign.RemoteDataScopeService;

@Configuration
@ConditionalOnBean(DataSource.class)
@AutoConfigureAfter(DataSourceAutoConfiguration.class)
@EnableConfigurationProperties(EasierMybatisProperties.class)
public class MybatisPlusConfiguration implements WebMvcConfigurer {

    /**
     * 增加请求参数解析器，对请求中的参数注入SQL 检查
     *
     * @param resolverList List<HandlerMethodArgumentResolver>
     */
    @Override
    public void addArgumentResolvers(List<HandlerMethodArgumentResolver> resolverList) {
        resolverList.add(new SqlFilterArgumentResolver());
    }

    /**
     * mybatis plus 拦截器配置
     *
     * @return EasierDefaultDataScopeHandle
     */
    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor(TenantLineInnerInterceptor tenantLineInnerInterceptor,
        TenantLineInnerInterceptor applicationLineInnerInterceptor, DataScopeInterceptor dataScopeInterceptor,
        OptimisticLockerInnerInterceptor optimisticLockerInnerInterceptor) {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();
        // 乐观锁
        interceptor.addInnerInterceptor(optimisticLockerInnerInterceptor);
        // 应用ID支持
        interceptor.addInnerInterceptor(applicationLineInnerInterceptor);
        // 注入多租户支持
        interceptor.addInnerInterceptor(tenantLineInnerInterceptor);
        // 数据权限
        interceptor.addInnerInterceptor(dataScopeInterceptor);
        // 分页支持
        PaginationInnerInterceptor paginationInnerInterceptor = new PaginationInnerInterceptor();
        paginationInnerInterceptor.setMaxLimit(1000L);
        interceptor.addInnerInterceptor(paginationInnerInterceptor);
        return interceptor;
    }

    /**
     * 创建租户维护处理器对象
     *
     * @return 处理后的租户维护处理器
     */
    @Bean
    @ConditionalOnMissingBean(name = "tenantLineInnerInterceptor")
    public TenantLineInnerInterceptor tenantLineInnerInterceptor(EasierTenantConfigProperties tenantConfigProperties) {
        TenantLineInnerInterceptor tenantLineInnerInterceptor = new TenantLineInnerInterceptor();
        tenantLineInnerInterceptor.setTenantLineHandler(new EasierTenantHandler(tenantConfigProperties));
        return tenantLineInnerInterceptor;
    }

    /**
     * 创建应用维护处理器对象
     *
     * @return 处理后的应用维护处理器
     */
    @Bean
    @ConditionalOnMissingBean(name = "applicationLineInnerInterceptor")
    public TenantLineInnerInterceptor
        applicationLineInnerInterceptor(EasierApplicationConfigProperties applicationConfigProperties) {
        TenantLineInnerInterceptor applicationLineInnerInterceptor = new TenantLineInnerInterceptor();
        applicationLineInnerInterceptor.setTenantLineHandler(new EasierApplicationHandler(applicationConfigProperties));
        return applicationLineInnerInterceptor;
    }

    /**
     * 数据权限拦截器
     *
     * @return DataScopeInterceptor
     */
    @Bean
    @ConditionalOnMissingBean
    @ConditionalOnClass(EasierUser.class)
    public DataScopeInterceptor dataScopeInterceptor(RemoteDataScopeService dataScopeService) {
        DataScopeInnerInterceptor dataScopeInnerInterceptor = new DataScopeInnerInterceptor();
        dataScopeInnerInterceptor.setDataScopeHandle(new EasierDefaultDatascopeHandle(dataScopeService));
        return dataScopeInnerInterceptor;
    }

    @Bean
    @Primary
    @ConditionalOnBean(DataScopeInterceptor.class)
    public DataScopeSqlInjector dataScopeSqlInjector() {
        return new DataScopeSqlInjector();
    }

    @Bean
    public DruidSqlLogFilter sqlLogFilter(EasierMybatisProperties properties) {
        return new DruidSqlLogFilter(properties);
    }

    /**
     * 审计字段自动填充
     *
     * @return {@link MetaObjectHandler}
     */
    @Bean
    public MybatisPlusMetaObjectHandler mybatisPlusMetaObjectHandler() {
        return new MybatisPlusMetaObjectHandler();
    }

    /**
     * 数据库方言配置
     *
     * @return DatabaseIdProvider
     */
    @Bean
    public DatabaseIdProvider databaseIdProvider() {
        VendorDatabaseIdProvider databaseIdProvider = new VendorDatabaseIdProvider();
        Properties properties = new Properties();
        properties.setProperty("SQL Server", "mssql");
        databaseIdProvider.setProperties(properties);
        return databaseIdProvider;
    }

    @Bean
    public JsonStringArrayTypeHandler jsonStringArrayTypeHandler() {
        return new JsonStringArrayTypeHandler();
    }

    @Bean
    public JsonLongArrayTypeHandler jsonLongArrayTypeHandler() {
        return new JsonLongArrayTypeHandler();
    }

    @Bean
    public OptimisticLockerInnerInterceptor optimisticLockerInnerInterceptor() {
        return new OptimisticLockerInnerInterceptor();
    }

}
