package com.yaoyicloud.easier.common.data.resolver;

import java.util.Map;
import java.util.Objects;

import com.yaoyicloud.easier.common.core.constant.SecurityConstants;
import com.yaoyicloud.easier.common.core.util.SpringContextHolder;
import com.yaoyicloud.easier.workspace.api.feign.RemoteParamService;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import lombok.experimental.UtilityClass;

/**
 * 参数解析器
 *
 * @author jimmy
 * @version 1.2.0
 * @date 2024/04/07 17:10
 */
@UtilityClass
public class ParamResolver {

    /**
     * 根据多个key 查询value 配置 结果使用hutool 的maputil 进行包装处理 MapUtil.getBool(result,key)
     *
     * @param key key
     * @return Map<String, Object>
     */
    public Map<String, Object> getMap(String... key) {
        // 校验入参是否合法
        if (Objects.isNull(key)) {
            throw new IllegalArgumentException("参数不合法");
        }

        RemoteParamService remoteParamService = SpringContextHolder.getBean(RemoteParamService.class);
        return remoteParamService.getByKeys(key, SecurityConstants.FROM_IN).getData();
    }

    /**
     * 根据key 查询value 配置
     *
     * @param key key
     * @param defaultVal 默认值
     * @return value
     */
    public Long getLong(String key, Long... defaultVal) {
        return checkAndGet(key, Long.class, defaultVal);
    }

    /**
     * 根据key 查询value 配置
     *
     * @param key key
     * @param defaultVal 默认值
     * @return value
     */
    public Integer getInt(String key, Integer... defaultVal) {
        return checkAndGet(key, Integer.class, defaultVal);
    }

    /**
     * 根据key 查询value 配置
     *
     * @param key key
     * @param defaultVal 默认值
     * @return value
     */
    public String getStr(String key, String... defaultVal) {
        return checkAndGet(key, String.class, defaultVal);
    }

    private <T> T checkAndGet(String key, Class<T> clazz, T... defaultVal) {
        // 校验入参是否合法
        if (StrUtil.isBlank(key) || defaultVal.length > 1) {
            throw new IllegalArgumentException("参数不合法");
        }

        RemoteParamService remoteParamService = SpringContextHolder.getBean(RemoteParamService.class);

        String result = remoteParamService.getByKey(key, SecurityConstants.FROM_IN).getData();

        if (StrUtil.isNotBlank(result)) {
            return Convert.convert(clazz, result);
        }

        if (defaultVal.length == 1) {
            return Convert.convert(clazz, defaultVal.clone()[0]);

        }
        return null;
    }

}
