package com.yaoyicloud.template;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.yaoyicloud.constant.enums.ReportType;
import com.yaoyicloud.tools.OfficeUtil;
import com.yaoyicloud.tools.PdfProcessor;
import com.yaoyicloud.tools.TocEntity;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTc;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STVerticalJc;

import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.HackLoopTableRenderPolicy;
import com.deepoove.poi.policy.RenderPolicy;
import com.deepoove.poi.template.ElementTemplate;
import com.deepoove.poi.util.TableTools;
import com.itextpdf.kernel.geom.LineSegment;
import com.itextpdf.kernel.geom.Rectangle;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfPage;
import com.itextpdf.kernel.pdf.PdfReader;
import com.itextpdf.kernel.pdf.canvas.parser.EventType;
import com.itextpdf.kernel.pdf.canvas.parser.PdfCanvasProcessor;
import com.itextpdf.kernel.pdf.canvas.parser.data.IEventData;
import com.itextpdf.kernel.pdf.canvas.parser.data.TextRenderInfo;
import com.itextpdf.kernel.pdf.canvas.parser.listener.SimpleTextExtractionStrategy;



/**
 * 模板方法模式, 定义一个导出流程的骨架
 *
 * @author lixuesong
 * @date 2024/05/30
 */
public abstract class AbstractReportExporter {

    /**
     * 报告导出
     * @param reportType 报告类型
     * @param data  数据
     * @param reportTempWordFile 模板位置
     * @param reportBastPath pdf导出文件位置
     * @return
     * @throws IOException
     */
    public final String exportReport(ReportType reportType, Map<String, Object> data, String reportTempWordFile,
                                     String reportBastPath) throws IOException {
        // 获取模板地址
        String basicPath = this.getBasicPath();
        String reportImagePath = this.getReportImagePath();
        String templatePath = this.getTemplate(reportType, basicPath);

       // Map<String, Object> data = this.fetchAndAssembleData(relationId, reportType);
        // 定义策略
        ConfigureBuilder builder = Configure.newBuilder();
        this.bindConfigure(builder);
        // 渲染数据
        this.renderTemplate(reportTempWordFile, templatePath, builder, data);
        String imagePath = this.imagePath();
        // word转pdf
        String convert = OfficeUtil.convert(reportTempWordFile, reportImagePath);
        String html = OfficeUtil.formatHtml(convert);
        String mediumFontPath = OfficeUtil.convertHtmlToPdf(html, reportBastPath, getFontPaths(), false, 1);
        // pdf页面调整
        Path path = Paths.get(reportBastPath);
        // 获取文件名和扩展名
        String fileName = path.getFileName().toString();
        int dotIndex = fileName.lastIndexOf('.');
        String baseName = (dotIndex == -1) ? fileName : fileName.substring(0, dotIndex);
        String extension = (dotIndex == -1) ? "" : fileName.substring(dotIndex);
        // 构建新文件名
        String newFileName = baseName + "1" + extension;
        // 构建完整新路径
        String newPdfPath = path.resolveSibling(newFileName).toString();
        return PdfProcessor.processPdf(reportBastPath, imagePath, mediumFontPath, 1);
    }

    protected abstract String getReportImagePath();

    protected abstract List<String> getFontPaths();

    /**
     * 获取模板地址
     *
     * @param reportType 类型
     * @return {@link String} 模板地址
     */
    protected abstract String getTemplate(ReportType reportType, String basicPath);

    /**
     * 配置绑定
     *
     * @param builder POI configureBuilder
     * @return {@link String} 模板地址
     */
    protected abstract void bindConfigure(ConfigureBuilder builder);


    protected  abstract  String imagePath();


    /**
     * 渲染数据
     *
     * @param templatePath 模板路径
     * @param builder 配置builder
     * @param data 数据
     * @throws IOException io异常
     */
    protected abstract void renderTemplate(String reportTempWordFile, String templatePath, ConfigureBuilder builder,
        Map<String, Object> data) throws IOException;

    protected abstract String getBasicPath() throws IOException;

    protected ArrayList<TocEntity> extractHeadings(String pdfPath) throws IOException {
        ArrayList<TocEntity> headings = new ArrayList<>();
        try (PdfDocument pdfDoc = new PdfDocument(new PdfReader(pdfPath))) {
            for (int i = 1; i <= pdfDoc.getNumberOfPages(); i++) {
                PdfPage page = pdfDoc.getPage(i);
                final int currentPage = i;
                new PdfCanvasProcessor(new HeadingDetectionStrategy(headings, pdfDoc, currentPage))
                    .processPageContent(page);
            }
        }
        return headings;
    }

    protected ArrayList<TocEntity> processHeadings(ArrayList<TocEntity> headings) {
        ArrayList<TocEntity> result = new ArrayList<>();
        if (headings == null || headings.isEmpty()) {
            return result;
        }
        for (int i = 0; i < headings.size(); i++) {
            TocEntity current = headings.get(i);

            if (!current.getTitle().matches("^\\d+\\..*")) {
                if (!result.isEmpty()) {
                    TocEntity last = result.get(result.size() - 1);
                    if (last.getPageNumber() == current.getPageNumber()) {
                        TocEntity combined = new TocEntity(last.getTitle() + current.getTitle(), last.getPageNumber(),
                            last.getLevel(), last.getRect(), last.getFontSize(), last.getParagraphHeight());
                        result.set(result.size() - 1, combined);
                        continue;
                    }
                }
            }
            if (current.getTitle().matches("^\\d+\\.")) {
                if (i + 1 < headings.size()) {
                    TocEntity next = headings.get(i + 1);
                    TocEntity combined = new TocEntity(current.getTitle() + next.getTitle(), current.getPageNumber(),
                        current.getLevel(), current.getRect(), current.getFontSize(), current.getParagraphHeight());
                    result.add(combined);
                    i++;
                    continue;
                }
            }
            result.add(current);
        }

        return result;
    }

    protected RenderPolicy getScoreRenderPolicy() {
        RenderPolicy policy = new HackLoopTableRenderPolicy() {
            @SuppressWarnings("checkstyle:NestedForDepth")
            @Override
            public void render(ElementTemplate eleTemplate, Object data, XWPFTemplate template) {
                super.render(eleTemplate, data, template);
                XWPFDocument doc = template.getXWPFDocument();
                for (XWPFTable table : doc.getTables()) {
                    boolean isTargetTable = false;
                    // 判断是否为目标表格：通过检查表格第一行是否包含货物明细相关表头关键词
                    List<String> headerKeywords = List.of("删除");
                    XWPFTableRow firstRow = table.getRow(0);
                    if (firstRow != null) {
                        for (XWPFTableCell cell : firstRow.getTableCells()) {
                            for (XWPFParagraph para : cell.getParagraphs()) {
                                for (XWPFRun run : para.getRuns()) {
                                    String text = run.text().trim();
                                    if (headerKeywords.contains(text)) {
                                        isTargetTable = true;
                                        break;
                                    }
                                }
                                if (isTargetTable) {
                                    break;
                                }
                            }
                            if (isTargetTable) {
                                break;
                            }
                        }
                    }
                    if (isTargetTable) {
                        // 删除表头（假设表头是第一行）
                        if (table.getRows().size() > 0) {
                            table.removeRow(0);
                        }
                        // 合并第一列相同内容单元格并居中
                        mergeAndCenterFirstColumn(table);
                    }
                }
            }

            private void mergeAndCenterFirstColumn(XWPFTable table) {
                if (table.getNumberOfRows() < 2) {
                    return;
                }

                int startRow = 0;
                String currentValue = getCellText(table.getRow(0)).get(0);

                for (int i = 1; i < table.getNumberOfRows(); i++) {
                    List<String> rowValues = getCellText(table.getRow(i));
                    if (rowValues.isEmpty()) {
                        continue;
                    }

                    String firstCellValue = rowValues.get(0);
                    if (firstCellValue.equals(currentValue)) {
                        // 继续查找相同值的行
                        continue;
                    } else {
                        // 合并从startRow到i-1行的第一列单元格
                        if (i - 1 > startRow) {
                            TableTools.mergeCellsVertically(table, 0, startRow, i - 1);
                            // 设置合并后单元格内容居中
                            setCellCenterAlignment(table.getRow(startRow).getCell(0));
                        }
                        startRow = i;
                        currentValue = firstCellValue;
                    }
                }

                // 处理最后一组相同值
                if (table.getNumberOfRows() - 1 > startRow) {
                    TableTools.mergeCellsVertically(table, 0, startRow, table.getNumberOfRows() - 1);
                    setCellCenterAlignment(table.getRow(startRow).getCell(0));
                }
            }
        };
        return policy;
    }

    private List<String> getCellText(XWPFTableRow row) {
        List<String> texts = new ArrayList<>();
        if (row == null) {
            return texts;
        }
        for (XWPFTableCell cell : row.getTableCells()) {
            StringBuilder sb = new StringBuilder();
            for (XWPFParagraph para : cell.getParagraphs()) {
                for (XWPFRun run : para.getRuns()) {
                    sb.append(run.text());
                }
            }
            texts.add(sb.toString().trim());
        }
        return texts;
    }

    private void setCellCenterAlignment(XWPFTableCell cell) {
        CTTc cttc = cell.getCTTc();
        CTTcPr ctPr = cttc.addNewTcPr();
        ctPr.addNewVAlign().setVal(STVerticalJc.CENTER);
    }

    private static class HeadingDetectionStrategy extends SimpleTextExtractionStrategy {
        private static final float LEVEL_1_FONT_SIZE = 16;
        private static final float LEVEL_2_FONT_SIZE = 12;
        private final ArrayList<TocEntity> headings;
        private final PdfDocument pdfDoc;
        private final int currentPage;

        HeadingDetectionStrategy(ArrayList<TocEntity> headings, PdfDocument pdfDoc, int currentPage) {
            this.headings = headings;
            this.pdfDoc = pdfDoc;
            this.currentPage = currentPage;
        }

        @Override
        public void eventOccurred(IEventData data, EventType type) {
            if (type == EventType.RENDER_TEXT) {
                TextRenderInfo info = (TextRenderInfo) data;
                float fontSize = info.getFontSize();

                if (fontSize == LEVEL_1_FONT_SIZE) {
                    addHeading(info, 1, pdfDoc, currentPage, headings);
                } else if (fontSize == LEVEL_2_FONT_SIZE) {
                    addHeading(info, 2, pdfDoc, currentPage, headings);
                }
            }

            super.eventOccurred(data, type);
        }

        private void addHeading(TextRenderInfo info, int level, PdfDocument pdfDoc, int i,
            ArrayList<TocEntity> headings) {
            String text = info.getText().trim();
            LineSegment ascentLine = info.getAscentLine();
            LineSegment descentLine = info.getDescentLine();
            float ascentY = ascentLine.getStartPoint().get(1);
            float descentY = descentLine.getStartPoint().get(1);
            float paragraphHeight = ascentY - descentY;

            Rectangle rect = info.getBaseline().getBoundingRectangle();
            if (!text.isEmpty()) {
                headings.add(new TocEntity(text, i, level, rect, paragraphHeight, info.getFontSize()

                ));
            }
        }

    }
}
