package com.yaoyicloud.tools;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Entities;
import org.jsoup.select.Elements;

import com.itextpdf.html2pdf.ConverterProperties;
import com.itextpdf.html2pdf.HtmlConverter;
import com.itextpdf.html2pdf.css.apply.impl.DefaultCssApplierFactory;
import com.itextpdf.html2pdf.resolver.font.DefaultFontProvider;
import com.itextpdf.io.font.FontProgramFactory;
import com.itextpdf.io.font.PdfEncodings;
import com.itextpdf.kernel.events.Event;
import com.itextpdf.kernel.events.IEventHandler;
import com.itextpdf.kernel.events.PdfDocumentEvent;
import com.itextpdf.kernel.font.PdfFont;
import com.itextpdf.kernel.font.PdfFontFactory;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfPage;
import com.itextpdf.kernel.pdf.PdfWriter;
import com.itextpdf.kernel.pdf.canvas.PdfCanvas;
import com.itextpdf.layout.Canvas;
import com.itextpdf.layout.element.Paragraph;
import com.itextpdf.layout.font.FontProvider;
import com.itextpdf.layout.properties.TextAlignment;
import com.itextpdf.styledxmlparser.css.media.MediaDeviceDescription;
import com.itextpdf.styledxmlparser.css.media.MediaType;

import fr.opensagres.poi.xwpf.converter.core.FileURIResolver;
import fr.opensagres.poi.xwpf.converter.core.ImageManager;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLConverter;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLOptions;

//import org.apache.poi.xwpf.converter.core.FileImageExtractor;
//import org.apache.poi.xwpf.converter.core.FileURIResolver;

/**
 * 使用poi+itextpdf进行word转pdf 先将word转成html，再将html转成pdf
 *
 * @author ：hewie
 * @date ：Created in 2020/2/27 22:41
 */
public class OfficeUtil {

    /**
     * 将docx转为html
     *
     * @param docxPath 输入docx文件路径
     * @return 生成的HTML内容
     */
    public static String convert(String docxPath, String imageDir) throws IOException {
        File imageDirFile = new File(imageDir);
        if (!imageDirFile.exists() && !imageDirFile.mkdirs()) {
            throw new IOException("无法创建图片目录: " + imageDir);
        }

        try (InputStream docxIn = new FileInputStream(docxPath);
             XWPFDocument document = new XWPFDocument(docxIn);
             ByteArrayOutputStream htmlOut = new ByteArrayOutputStream()) {

            // 自定义 ImageManager：禁止生成 word/media 目录，强制使用 imageDir 根目录
            ImageManager imageManager = new ImageManager(imageDirFile, "") {
                @Override
                public String resolve(String uri) {

                    return new File(imageDir, uri).getAbsolutePath().replace("/", "\\").toString();
                }

            };

            XHTMLOptions options = XHTMLOptions.create()
                    .setImageManager(imageManager)
                    .URIResolver(new FileURIResolver(imageDirFile) {
                        @Override
                        public String resolve(String uri) {
                            // 去除 word/media/ 前缀
                            String filename = uri.replace("word/media/", "");
                            return new File(imageDirFile, filename)
                                    .getAbsolutePath()
                                    .replace("/", "\\");
                        }
                    });

            XHTMLConverter.getInstance().convert(document, htmlOut, options);
            return htmlOut.toString("UTF-8");
        }
    }


    /**
     * 使用jsoup规范化html
     *
     * @param html html内容
     * @return 规范化后的html
     */
    public static String formatHtml(String html) {
        Document doc = Jsoup.parse(html);

        // 1. 添加基础样式
        String baseCss = "@page { size: A4; } "
                +   "table { "
                +  "  width: 100%; "
                +  "  border-collapse: collapse; "
                +  "  page-break-inside: auto; "
                +   "} "
                +   "td, th { "
                +   "  page-break-inside: avoid; "  // 尽量保持单元格不分页
                +   "  -fs-table-paginate: paginate; "  // 允许分页
                +   "  background-clip: padding-box; "  // 确保背景色覆盖
                +   "  -webkit-print-color-adjust: exact; "  // 确保打印时颜色准确
                +   "}";
        doc.head().appendElement("style").text(baseCss);
        Elements images = doc.select("img");
        Element firstImg = images.first();

        // 4. 删除第一个img元素
        firstImg.remove();
        Elements div = doc.select("div");
        div.attr("style", "page-break-before:always; ");

        Elements table = doc.select("table");
        String style = table.attr("style");
        style += "width:100%;";
        table.attr("style", style);

        // 处理 td 和 p 标签
        Elements tds = doc.select("td");
        for (Element td : tds) {
            Elements ps = td.select("p");
            if (ps.size() > 1) {
                for (int i = 1; i < ps.size(); i++) {
                    ps.get(i).remove();
                }
                Element p = ps.first();
                String pStyle = p.attr("style");
                pStyle += " display: table-cell; vertical-align: middle;";
                p.attr("style", pStyle);
            }
            if (ps.size() > 0) {
                Element p = ps.first();
                String oriPstyle = p.attr("style");
                p.attr("style", oriPstyle + " margin-left:0.5em;");
            }
            // 修改单元格样式 - 关键修改点
            String oristyle = td.attr("style");
            oristyle = (oristyle == null) ? "" : oristyle;
            // 移除可能干扰分页的固定高度设置
            oristyle += " border-collapse: collapse; border: 0.75pt solid #E3EDFB;";
            oristyle += " -fs-table-paginate: paginate;"; // 允许分页
            oristyle += " background-clip: padding-box;"; // 确保背景完整
            td.attr("style", oristyle);
        }

        Elements elementsWithMargin = doc.select("span.X1.X2");
        for (Element el : elementsWithMargin) {
            String x1Sytle = el.attr("style");
            x1Sytle = x1Sytle.replaceAll("margin-left:\\s*[^;]+;?", "");
            if (!el.text().contains("重要声明")) {
                x1Sytle += "color:#1677ff; ";
            }

            el.attr("style", x1Sytle);
        }
        Elements select = doc.select("p.X1.X2");
        for (Element el : select) {
            String style1 = el.attr("style");
            style1 += "page-break-before:always;";
            el.attr("style", style1);
        }

        Elements selectp = doc.select("p");
        for (Element el : selectp) {
            if (el.text().trim() != null) {
                String styleP = el.attr("style");
                styleP += " margin-top:6pt; margin-bottom:6pt";
                el.attr("style", styleP);
            }
        }
        Elements x3SpanStyle = doc.select("p.X1.X3");
        for (Element el : x3SpanStyle) {
            // 原有样式修改
            String x3style = el.attr("style");
            x3style = x3style.replaceAll("margin-bottom:0.0pt", "margin-bottom:7pt");
            el.attr("style", x3style);
            Element nextP = el.nextElementSibling();
            if (nextP != null && "p".equals(nextP.tagName())) {
                if (nextP.text().trim().isEmpty()) {  // 判断是否为空内容
                    nextP.remove();  // 删除空p标签
                }
            }
        }
        doc.outputSettings().syntax(Document.OutputSettings.Syntax.xml);
        doc.outputSettings().escapeMode(Entities.EscapeMode.xhtml);
        doc.head().prepend("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        return doc.html();
    }



    public static String convertHtmlToPdf(String htmlString, String outputPdfPath, List<String> fontPaths,
        boolean simpleFooter, int coverPageCount) throws IOException {
        File outputFile = new File(outputPdfPath);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try (OutputStream os = new FileOutputStream(outputFile); PdfWriter writer = new PdfWriter(os);
            PdfDocument pdf = new PdfDocument(writer)) {
            String boldFont = null;
            String regularFont = null;
            for (String path : fontPaths) {
                if (path.contains("bold")) {
                    boldFont = path;
                } else if (path.contains("regular")) {
                    regularFont = path;
                }
            }
            pdf.addEventHandler(PdfDocumentEvent.END_PAGE, new IEventHandler() {

                @Override
                public void handleEvent(Event event) {
                    PdfDocumentEvent docEvent = (PdfDocumentEvent) event;
                    PdfPage page = docEvent.getPage();
                    int currentPageNumber = docEvent.getDocument().getPageNumber(page);

                    if (currentPageNumber <= coverPageCount) {
                        return;
                    }

                    PdfCanvas pdfCanvas =
                        new PdfCanvas(page.newContentStreamAfter(), page.getResources(), docEvent.getDocument());
                    Canvas canvas = new Canvas(pdfCanvas, page.getPageSize());

                    float x = (page.getPageSize().getLeft() + page.getPageSize().getRight()) / 2;
                    float y = page.getPageSize().getBottom() + 20;

                    Paragraph p = simpleFooter
                        ? new Paragraph(String.valueOf(currentPageNumber - coverPageCount)).setFontSize(9).setMargin(0)
                            .setFixedPosition(x, y, 50).setTextAlignment(TextAlignment.CENTER)
                        : new Paragraph("第 " + String.valueOf(currentPageNumber - coverPageCount) + " 页").setFontSize(9).setMargin(0)
                            .setFixedPosition(x, y, 50).setTextAlignment(TextAlignment.CENTER); // .setFontColor(new Color());

                    canvas.add(p);
                    canvas.close();
                }
            });
            FontProvider fontProvider = new DefaultFontProvider(false, false, false);
            PdfFont microsoft = PdfFontFactory.createFont(FontProgramFactory.createFont(boldFont));
            fontProvider.addFont(microsoft.getFontProgram(), PdfEncodings.IDENTITY_H);
            PdfFont microsoft1 = PdfFontFactory.createFont(FontProgramFactory.createFont(regularFont));
            fontProvider.addFont(microsoft1.getFontProgram(), PdfEncodings.IDENTITY_H);
            ConverterProperties converterProps = new ConverterProperties();
            converterProps.setFontProvider(fontProvider);
            converterProps.setMediaDeviceDescription(new MediaDeviceDescription(MediaType.PRINT));
            converterProps.setCssApplierFactory(new DefaultCssApplierFactory());

            // 先进行 HTML 到 PDF 的转换
            HtmlConverter.convertToPdf(htmlString, pdf, converterProps);

            return regularFont;
        }
    }

}
